module fifo_port_uart(
       input        clk50M,
		 input        rst_n,
		 input [7:0]  data,
		 input        write,
		 output       wr_full,
		 output       rd_empty,
		 input        read,
		 output [7:0] q,
		 input        uart_rxd,
		 output       uart_txd
);
precise_divider
#(
	//DEVIDE_CNT = 85.89934592 * fo
//	parameter		DEVIDE_CNT = 32'd351843721	//256000bps * 16
//	parameter		DEVIDE_CNT = 32'd175921860	//128000bps * 16
//	.DEVIDE_CNT(32'd158329674)                //115200bps * 16
//   .DEVIDE_CNT(32'd79164837)                 //57600bps  * 16
	.DEVIDE_CNT(32'd13194140)	//9600bps * 16
)
precise_divider_u
(
	//global clock
	.clk(clk50M),
	.rst_n(rst_n),
	
	//user interface
	//.divide_clk(),
	.divide_clken(clken_16bps)
);
wire [7:0] txfifo_q_w;
reg [7:0]  txfifo_q_r;
wire       txfifo_rd_w,txfifo_empty_w;
uart_fifo #(
  .DEPTH( 1024 ),
  .DATA_W( 8 )
) uart_txfifo (
  .clk( clk50M ),
  .rst( ~rst_n ),
  .w_req( write ),
  .w_data( data ),
  .r_req( txfifo_rd_w ),
  .r_data( txfifo_q_w ),
  .empty( txfifo_empty_w ),
  .full( wr_full )
);
wire clken_16bps,txd_flag_w;
wire txd_en_w;
assign txd_en_w = txfifo_rd_r;
uart_transfer uart_transfer_u(
	 .clk(clk50M),
	 .rst_n(rst_n),
	 .clken_16bps(clken_16bps),
	 .txd(uart_txd),		
	 .txd_en(txd_en_w),		
    .txd_data(txfifo_q_r),
	 .txd_flag(txd_flag_w)
	 );
wire rxd_flag_w;
wire [7:0] rxd_data_w;
uart_receiver uart_receiver_u(//gobal clock
	.clk(clk50M),
	.rst_n(rst_n),
	.clken_16bps(clken_16bps),
	.rxd(uart_rxd),
	.rxd_flag(rxd_flag_w),
	.rxd_data(rxd_data_w)	
	               	);
uart_fifo #(
  .DEPTH( 1024 ),
  .DATA_W( 8 )
) uart_rxfifo (
  .clk( clk50M ),
  .rst( ~rst_n ),
  .w_req( rxd_flag_w ),
  .w_data( rxd_data_w ),
  .r_req( read ),
  .r_data( q ),
  .empty( rd_empty )
);
reg tx_state;
reg txfifo_rd_r;
assign txfifo_rd_w= (~txfifo_empty_w) & (~tx_state);
always@(posedge clk50M or negedge rst_n)
begin
if(!rst_n)
    begin
	 tx_state    <= 1'b0;
	 txfifo_rd_r <= 1'b0;
	 txfifo_q_r <= 8'd0;
	 end
else
    begin
	 txfifo_rd_r   <= txfifo_rd_w;
	 txfifo_q_r <= txfifo_rd_w?txfifo_q_w:txfifo_q_r;
	 case(tx_state)
	 1'b0:tx_state <= txfifo_empty_w?tx_state:1'b1;
	 1'b1:tx_state <= txd_flag_w?1'b0:tx_state;
	 endcase
	 end
end
endmodule 