//****************************************Copyright (c)***********************************//
//技术支持：www.openedv.com
//淘宝店铺：http://openedv.taobao.com
//关注微信公众平台微信号："正点原子"，免费获取FPGA & STM32资料。
//版权所有，盗版必究。
//Copyright(C) 正点原子 2018-2028
//All rights reserved
//----------------------------------------------------------------------------------------
// File name:           temp_disp
// Last modified Date:  2018-03-30 14:45:44
// Last Version:        V1.0
// Descriptions:        温度传感器显示
//----------------------------------------------------------------------------------------
// Created by:          正点原子
// Created date:        2018-03-30 14:46:40
// Version:             V1.0
// Descriptions:        The original version
//
//----------------------------------------------------------------------------------------
//****************************************************************************************//

module temp_disp(
    input            sys_clk     ,    //输入的系统时钟
    input            sys_rst_n   ,    //输入的复位信号
    inout            dq          ,    //ds18b20温度传感器单总线
    output  [5:0]    sel         ,    //输出数码管位选信号
    output  [7:0]    seg_led     ,     //输出数码管段选信号
	 output           uart_txd
);

//parameter define
parameter POINT = 6'b000100;          // 数码管小数点的位置

//wire define
wire    [19:0]  temp_data;            // 温度数值
wire            sign;                 // 符号位

wire   [3:0]              data0    ;        // 个位数
wire   [3:0]              data1    ;        // 十位数
wire   [3:0]              data2    ;        // 百位数
wire   [3:0]              data3    ;        // 千位数
reg [7:0] tx_data;
reg       tx_valid;
reg [31:0] cnt;//计数器计数到24999999，也就是每秒钟循环2次，发送两次完整的字符串
reg [3:0] d0_r,d1_r,d2_r,d3_r;//寄存个十百千位，d0_r对应个位
//*****************************************************
//**                    main code
//*****************************************************

//例化动态数码管驱动模块
seg_led u_seg_led(
    //module clock
    .clk           (sys_clk  ),       // 时钟信号
    .rst_n         (sys_rst_n),       // 复位信号
    //seg_led interface
    .seg_sel       (sel      ),       // 位选
    .seg_led       (seg_led  ),       // 段选
    //user interface
	.data0(data0)    ,        // 个位数
	.data1(data1)    ,        // 十位数
	.data2(data2)    ,        // 百位数
	.data3(data3)    ,        // 千位数	 
    .data          (temp_data),       // 显示的数值
    .point         (POINT    ),       // 小数点具体显示的位置,从高到低,高电平有效
    .en            (1'b1     ),       // 数码管使能信号
    .sign          (sign     )        // 符号位（高电平显示“-”号）
);

//例化DS18B20驱动模块
ds18b20_dri u1_ds18b20_dri(
    //module clock
    .clk          (sys_clk  ),        // 时钟信号（50MHz）
    .rst_n        (sys_rst_n),        // 复位信号
    //user interface
    .dq           (dq       ),        // DS18B20的DQ引脚数据
    .temp_data    (temp_data),        // 转换后得到的温度值
    .sign         (sign     )         // 符号位
);
fifo_port_uart u_fifo_port_uart_0(
 		.clk50M(sys_clk),
		.rst_n(sys_rst_n),
		.data(tx_data),
		.write(tx_valid),
		.uart_txd(uart_txd)
);
always@(posedge sys_clk or negedge sys_rst_n)
begin
if(!sys_rst_n)
    begin
	 d0_r <= 4'h0;
	 d1_r <= 4'h0;
	 d2_r <= 4'h0;
	 d3_r <= 4'h0;
	 cnt <= 0;
	 tx_data <= 8'h00;
	 tx_valid <= 1'b0;
	 end
else
    begin
	 if(cnt == 0)
	     begin
		  d0_r <= data0;
	     d1_r <= data1;
	     d2_r <= data2;
	     d3_r <= data3;
		  tx_data <= 8'h00;
		  end
	 else
	     begin
		  d0_r <= d0_r;
	     d1_r <= d1_r ;
	     d2_r <= d2_r ;
	     d3_r <= d3_r ;		  
		  end
	 cnt <= (cnt >= 24999999)?0:cnt+1;
	 if(cnt == 1)
	     tx_data <= sign?"-":"+";
	 else if(cnt == 2)
	     tx_data <= d3_r+8'h30;//数字转成其对应的ascii码
	 else if(cnt == 3)
	     tx_data <= d2_r+8'h30;//数字转成其对应的ascii码
	 else if(cnt == 4)
	     tx_data <= ".";       //小数点
	 else if(cnt == 5)
	     tx_data <= d1_r+8'h30;//数字转成其对应的ascii码
	 else if(cnt == 6)
	     tx_data <= d0_r+8'h30;//数字转成其对应的ascii码
	 else if(cnt == 7)
	     tx_data <= "\n";      //回车换行
	 else
	     tx_data <= 8'h00;
	 tx_valid <= (cnt >= 1 && cnt <= 7);
	 end
end
endmodule 